@@ -5,10 +5,10 @@ use base 'Module::Setup::Plugin';
 
 sub register {
     my($self, ) = @_;
-    $self->add_trigger( befor_dump_config => \&befor_dump_config );
+    $self->add_trigger( before_dump_config => \&before_dump_config );
 }
 
-sub befor_dump_config {
+sub before_dump_config {
     my($self, $config) = @_;
 
     $config->{author} ||= 'Default Name';
