@@ -56,3 +56,6 @@ __DATA__
   pm_to_blib
   MANIFEST
   Makefile.old
+  nytprof.out
+  MANIFEST.bak
+  *.sw[po]
